﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text.RegularExpressions;
using NSpec;

class check_namespaces : nspec {
    static string getEntitasProjectDir() {
        var dirInfo = new DirectoryInfo(Directory.GetCurrentDirectory());
        return dirInfo.FullName;
    }

    static Dictionary<string, string> getSourceFiles(string path) {
        return Directory.GetFiles(path, "*.cs", SearchOption.AllDirectories)
            .Where(p => !p.Contains("Generated" + Path.DirectorySeparatorChar) &&
                !p.Contains("Libraries" + Path.DirectorySeparatorChar) &&
                !p.Contains("Tests" + Path.DirectorySeparatorChar) &&
                !p.Contains("Examples" + Path.DirectorySeparatorChar) &&
                !p.Contains("Readme" + Path.DirectorySeparatorChar) &&
                !p.Contains("bin" + Path.DirectorySeparatorChar) &&
                !p.Contains("obj" + Path.DirectorySeparatorChar) &&
                !p.Contains("AssemblyInfo.cs") &&
                !p.Contains("Commands.cs") &&
                !p.Contains("Program.cs"))
            .ToDictionary(p => p, p => File.ReadAllText(p));
    }

    void when_running_test() {
        it["checks namespaces"] = () => {
            var entitasDir = getEntitasProjectDir();
            var sourceFiles = getSourceFiles(entitasDir);

            sourceFiles.Count.should_be_greater_than(60);
            sourceFiles.Count.should_be_less_than(100);

            const string namespacePattern = @"(?:^namespace)\s.*\b";
            string expectedNamespacePattern = string.Format(@".+?[^\{0}]*", Path.DirectorySeparatorChar);

            foreach (var file in sourceFiles) {
                var fileName = file.Key.Replace(entitasDir + Path.DirectorySeparatorChar, string.Empty);
                var expectedNamespace = Regex.Match(fileName, expectedNamespacePattern)
                        .ToString()
                        .Replace("namespace ", string.Empty)
                        .Trim();

                var foundNamespace = Regex.Match(file.Value, namespacePattern, RegexOptions.Multiline)
                        .ToString()
                        .Replace("namespace ", string.Empty)
                        .Trim();

                if (foundNamespace != expectedNamespace) {
                    Console.WriteLine(fileName + " exp: '" + expectedNamespace + "' was: '" + foundNamespace + "'");
                }

                foundNamespace.should_be(expectedNamespace);
            }
        };
    }
}

